@@ -10,13 +10,13 @@ plinks = function( bdgraph.obj, round = 3 )
 		p             <- nrow( bdgraph.obj $ last_graph )
 		pvec          <- c( rep( 0, p * ( p - 1 ) / 2 ) )
 	   
-		for ( i in 1 : length( sample_graphs ) )
+		for( i in 1 : length( sample_graphs ) )
 		{
 			inp       <- which( unlist( strsplit( as.character( sample_graphs[i] ), "" ) ) == 1 )
 			pvec[inp] <- pvec[inp] + graph_weights[i]
 		}
 		
-		dimlab  <- colnames( bdgraph.obj $ last_graph ) # lastG
+		dimlab  <- colnames( bdgraph.obj $ last_graph ) 
 		p_links <- matrix( 0, p, p, dimnames = list( dimlab, dimlab ) )
 		p_links[ upper.tri(p_links) ] <- pvec / sum( graph_weights )
 	}
