@@ -1,7 +1,18 @@
+// ----------------------------------------------------------------------------|
+//     Copyright (C) 2012-2016 Mohammadi A. and Wit C. E.
+//
+//     This file is part of BDgraph package.
+//
+//     BDgraph is free software: you can redistribute it and/or modify it under 
+//     the terms of the GNU General Public License as published by the Free 
+//     Software Foundation; see <https://cran.r-project.org/web/licenses/GPL-3>.
+//
+//     Maintainer:
+//     Abdolreza Mohammadi: a.mohammadi@rug.nl or a.mohammadi@uvt.nl
+// ----------------------------------------------------------------------------|
 #include <sstream>
 #include <string>        // std::string, std::to_string
 #include <vector>        // for using vector
-#include <limits>        // for std::numeric_limits<long double>::max()
 #include <R.h>
 #include <Rmath.h>
 #include "matrix.h"
@@ -11,22 +22,23 @@
 using namespace std;
 
 extern "C" {
-/*
- * birth-death MCMC for Gaussian copula Graphical models  
- * Based on Double Metropolis-Hastings
- * it is for Bayesian model averaging
-*/
+// ----------------------------------------------------------------------------|
+// birth-death MCMC for Gaussian copula Graphical models  
+// Based on Double Metropolis-Hastings
+// it is for Bayesian model averaging
+// ----------------------------------------------------------------------------|
 void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], double p_links[],
-			 int *b, int *b_star, double D[], double Ds[], double *threshold )
+			 int *b, int *b_star, double D[], double Ds[], int *print )
 {
+	int print_c = *print;
 	int iteration = *iter, burn_in = *burnin, b1 = *b;
 	
 	int index_selected_edge, selected_edge_i, selected_edge_j, selected_edge_ij, counter;
 
-	double Dsijj, Dsjj, Dsij, K022, sigmaj11, threshold_C = *threshold;
-	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
+	double Dsijj, Dsjj, Dsij, K022, weight_C;
+	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
 	vector<double> sigma( pxp ); 
 	vector<double> copyK( pxp ); 
@@ -35,7 +47,7 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 
 	int qp = dim * ( dim - 1 ) / 2;
 	double sum_weights = 0.0;
-	long double rate, sum_rates; 
+	double log_rate, sum_rates; 
 
 	vector<double> p_links_Cpp( pxp, 0.0 ); 
 	vector<double> K_hat_Cpp( pxp, 0.0 ); 
@@ -52,7 +64,7 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 	}
 
 	// For finding the index of rates 
-	vector<long double> rates( qp );
+	vector<double> rates( qp );
 	vector<int> index_rates_row( qp );
 	vector<int> index_rates_col( qp );
 	counter = 0;
@@ -96,20 +108,18 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 	vector<double> K_dmh( pxp );              // for double Metropolis-Hastings
 	double logH_ij, logI_p, Dij, Djj, Dijj;   // for double Metropolis-Hastings
 
-	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
-
 	GetRNGstate();
 	// main loop for birth-death MCMC sampling algorithm ----------------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
 	{
-		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
+		if( ( i_mcmc + 1 ) % print_c == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
 		// STEP 1: copula -----------------------------------------------------|
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
 		get_Ts( Ds, Ts, &inv_Ds[0], &copy_Ds[0], &dim );
 		
 		// sampling from K and sigma for double Metropolis-Hastings
-		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
 		counter = 0;
 		// STEP 1: calculating birth and death rates --------------------------|		
@@ -139,12 +149,26 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
 
-				rate = ( G[ij] ) ? exp( logH_ij - logI_p ) : exp( logI_p - logH_ij );
-				
-				rates[counter++] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
+				log_rate = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );				
+				rates[counter++] = ( log_rate < 0.0 ) ? exp( log_rate ) : 1.0;			
 			}
 		}
 		
+//----- saving result ---------------------------------------------------------|	
+		if( i_mcmc >= burn_in )
+		{
+			weight_C = 1.0 / sum_rates;
+			
+			// K_hat_Cpp[i] += K[i] / sum_rates;
+			F77_NAME(daxpy)( &pxp, &weight_C, &K[0], &one, &K_hat_Cpp[0], &one );
+			
+			for( i = 0; i < pxp ; i++ )
+				if( G[i] ) p_links_Cpp[i] += weight_C;
+			
+			sum_weights += weight_C;
+		} 
+//----- End of saving result --------------------------------------------------|	
+		
 		// Selecting an edge based on birth and death rates
 		select_edge( &rates[0], &index_selected_edge, &sum_rates, &qp );
 		selected_edge_i = index_rates_row[ index_selected_edge ];
@@ -167,19 +191,7 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 		}
 
 		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
-		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
-
-		// saving result ------------------------------------------------------|	
-		if( i_mcmc >= burn_in )
-		{
-			for( i = 0; i < pxp ; i++ )
-			{
-				K_hat_Cpp[i]   += K[i] / sum_rates;
-				if( G[i] ) p_links_Cpp[i] += 1.0 / sum_rates;
-			}
-			
-			sum_weights += 1.0 / sum_rates;
-		} // End of saving result ---------------------------------------------|	
+		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
@@ -190,27 +202,28 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 	}
 }
    
-/*
- * birth-death MCMC for Gaussian copula Graphical models  
- * Based on Double Metropolis-Hastings
- * it is for maximum a posterior probability estimation (MAP)
-*/
+// ----------------------------------------------------------------------------|
+// birth-death MCMC for Gaussian copula Graphical models  
+// Based on Double Metropolis-Hastings
+// it is for maximum a posterior probability estimation (MAP)
+// ----------------------------------------------------------------------------|
 void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
-			 int *b, int *b_star, double D[], double Ds[], double *threshold )
+			 int *b, int *b_star, double D[], double Ds[], int *print )
 {
+	int print_c = *print;
 	int iteration = *iter, burn_in = *burnin, b1 = *b;
-	int counterallG = 0;
+	int count_all_g = 0;
 	string string_g;
 	vector<string> sample_graphs_C( iteration - burn_in );
 	
 	int index_selected_edge, selected_edge_i, selected_edge_j, selected_edge_ij, counter, size_sample_graph = *size_sample_g;
 	bool this_one;
 
-	double Dsijj, Dsjj, Dsij, K022, threshold_C = *threshold;
-	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
+	double Dsijj, Dsjj, Dsij, K022, weight_C;
+	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
 	vector<double> sigma( pxp ); 
 	vector<double> copyK( pxp ); 
@@ -220,7 +233,7 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 	int qp = dim * ( dim - 1 ) / 2;
 	vector<char> char_g( qp );              // char string_g[pp];
 	double sum_weights = 0.0;
-	long double rate, sum_rates; 
+	double log_rate, sum_rates; 
 	
 	vector<double> K121( 4 ); 																
 
@@ -234,7 +247,7 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 	}
 
 	// For finding the index of rates 
-	vector<long double> rates( qp );
+	vector<double> rates( qp );
 	vector<int> index_rates_row( qp );
 	vector<int> index_rates_col( qp );
 	counter = 0;
@@ -279,20 +292,18 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 	double logH_ij, logI_p, Dij, Djj, Dijj;   // for double Metropolis-Hastings
 	// ----------------------------------------
 
-	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
-
 	GetRNGstate();
 	// main loop for birth-death MCMC sampling algorithm ----------------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
 	{
-		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
+		if( ( i_mcmc + 1 ) % print_c == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
 		// STEP 1: copula -----------------------------------------------------|
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
 		get_Ts( Ds, Ts, &inv_Ds[0], &copy_Ds[0], &dim );
 		
 		// sampling from K and sigma for double Metropolis-Hastings
-		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
 		counter = 0;
 		// STEP 1: calculating birth and death rates --------------------------|		
@@ -322,14 +333,47 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
 
-				rate = ( G[ij] ) ? exp( logH_ij - logI_p ) : exp( logI_p - logH_ij );
-				
-				rates[counter] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
+				log_rate = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );				
+				rates[counter] = ( log_rate < 0.0 ) ? exp( log_rate ) : 1.0;			
 				
 				char_g[counter] = G[ij] + '0'; 
 				counter++; 
 			}
 		}
+
+//----- saving result ---------------------------------------------------------|	
+		if( i_mcmc >= burn_in )
+		{
+			weight_C = 1.0 / sum_rates;
+			
+			//for( i = 0; i < pxp; i++ ) K_hat[i] += K[i] / sum_rates;
+			F77_NAME(daxpy)( &pxp, &weight_C, &K[0], &one, &K_hat[0], &one );			
+
+			string_g = string( char_g.begin(), char_g.end() );	
+			all_weights[count_all_g] = weight_C;
+			
+			this_one = false;
+			for( i = 0; i < size_sample_graph; i++ )
+				if( sample_graphs_C[i] == string_g )
+				{
+					graph_weights[i] += all_weights[count_all_g];
+					all_graphs[count_all_g] = i;
+					this_one = true;
+					break;
+				} 
+			
+			if( !this_one || size_sample_graph == 0 )
+			{
+				sample_graphs_C[size_sample_graph] = string_g;
+				graph_weights[size_sample_graph]   = all_weights[count_all_g];
+				all_graphs[count_all_g]            = size_sample_graph; 
+				size_sample_graph++;				
+			}
+			
+			count_all_g++; 
+			sum_weights += weight_C;
+		} 
+//----- End of saving result --------------------------------------------------|	
 		
 		// Selecting an edge based on birth and death rates
 		select_edge( &rates[0], &index_selected_edge, &sum_rates, &qp );
@@ -353,37 +397,7 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 		}
 
 		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
-		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
-
-		// saving result ------------------------------------------------------|	
-		if( i_mcmc >= burn_in )
-		{
-			for( i = 0; i < pxp ; i++ ) K_hat[i] += K[i] / sum_rates;	
-
-			string_g = string( char_g.begin(), char_g.end() );	
-			all_weights[counterallG] = 1.0 / sum_rates;
-			
-			this_one = false;
-			for( i = 0; i < size_sample_graph; i++ )
-				if( sample_graphs_C[i] == string_g )
-				{
-					graph_weights[i] += all_weights[counterallG];
-					all_graphs[counterallG] = i;
-					this_one = true;
-					break;
-				} 
-			
-			if( !this_one || size_sample_graph == 0 )
-			{
-				sample_graphs_C[size_sample_graph] = string_g;
-				graph_weights[size_sample_graph]   = all_weights[counterallG];
-				all_graphs[counterallG]          = size_sample_graph; 
-				size_sample_graph++;				
-			}
-			
-			counterallG++; 
-			sum_weights += 1.0 / sum_rates;
-		} // End of saving result ---------------------------------------------|	
+		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
@@ -399,23 +413,24 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 		K_hat[i] /= sum_weights;
 }
            
-/*
- * Multiple birth-death MCMC for Gaussian copula Graphical models  
- * Based on Double Metropolis-Hastings
- * it is for Bayesian model averaging
-*/
+// ----------------------------------------------------------------------------|
+// Multiple birth-death MCMC for Gaussian copula Graphical models  
+// Based on Double Metropolis-Hastings
+// it is for Bayesian model averaging
+// ----------------------------------------------------------------------------|
 void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], double p_links[],
-			 int *b, int *b_star, double D[], double Ds[], double *threshold, int *multi_update )
+			 int *b, int *b_star, double D[], double Ds[], int *multi_update, int *print )
 {
+	int print_c = *print;
 	int iteration = *iter, burn_in = *burnin, b1 = *b;
 	int multi_update_C = *multi_update;
 	
 	int selected_edge_i, selected_edge_j, selected_edge_ij, counter;
 
-	double Dsijj, Dsjj, Dsij, K022, threshold_C = *threshold;
-	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
+	double Dsijj, Dsjj, Dsij, K022, weight_C;
+	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
 	vector<double> sigma( pxp ); 
 	vector<double> copyK( pxp ); 
@@ -424,7 +439,7 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts
 
 	int qp = dim * ( dim - 1 ) / 2;
 	double sum_weights = 0.0;
-	long double rate, sum_rates; 
+	double log_rate, sum_rates; 
 
 	vector<double> p_links_Cpp( pxp, 0.0 ); 
 	vector<double> K_hat_Cpp( pxp, 0.0 ); 
@@ -441,7 +456,7 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts
 	}
 
 	// For finding the index of rates 
-	vector<long double> rates( qp );
+	vector<double> rates( qp );
 	vector<int> index_rates_row( qp );
 	vector<int> index_rates_col( qp );
 	counter = 0;
@@ -488,20 +503,18 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts
 	int size_index = multi_update_C;
 	vector<int> index_selected_edges( multi_update_C );
 
-	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
-
 	GetRNGstate();
 	// main loop for multiple birth-death MCMC sampling algorithm -------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc += size_index )
 	{
-		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
+		if( ( i_mcmc + 1 ) % print_c == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
 		// STEP 1: copula -----------------------------------------------------|
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
 		get_Ts( Ds, Ts, &inv_Ds[0], &copy_Ds[0], &dim );
 		
 		// sampling from K and sigma for double Metropolis-Hastings
-		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
 		counter = 0;
 		// STEP 1: calculating birth and death rates --------------------------|		
@@ -531,11 +544,25 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
 
-				rate = ( G[ij] ) ? exp( logH_ij - logI_p ) : exp( logI_p - logH_ij );
-				
-				rates[counter++] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
+				log_rate = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );				
+				rates[counter++] = ( log_rate < 0.0 ) ? exp( log_rate ) : 1.0;			
 			}
 		}
+
+//----- saving result ---------------------------------------------------------|	
+		if( i_mcmc >= burn_in )
+		{
+			weight_C = 1.0 / sum_rates;
+			
+			// K_hat_Cpp[i] += K[i] / sum_rates;
+			F77_NAME(daxpy)( &pxp, &weight_C, &K[0], &one, &K_hat_Cpp[0], &one );
+			
+			for( i = 0; i < pxp ; i++ )
+				if( G[i] ) p_links_Cpp[i] += weight_C;
+			
+			sum_weights += weight_C;
+		} 
+//----- End of saving result --------------------------------------------------|	
 		
 		// Selecting multiple edges based on birth and death rates
 		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &qp );
@@ -563,19 +590,7 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts
 		}
 		
 		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
-		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
-
-		// saving result ------------------------------------------------------|	
-		if( i_mcmc >= burn_in )
-		{
-			for( i = 0; i < pxp ; i++ )
-			{
-				K_hat_Cpp[i] += K[i] / sum_rates;
-				if( G[i] ) p_links_Cpp[i] += 1.0 / sum_rates;
-			}
-			
-			sum_weights += 1.0 / sum_rates;
-		} // End of saving result ---------------------------------------------|	
+		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
@@ -586,28 +601,29 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts
 	}
 }
     
-/*
- * Multiple birth-death MCMC for Gaussian copula Graphical models  
- * Based on Double Metropolis-Hastings
- * it is for maximum a posterior probability estimation (MAP)
-*/
+// ----------------------------------------------------------------------------|
+// Multiple birth-death MCMC for Gaussian copula Graphical models  
+// Based on Double Metropolis-Hastings
+// it is for maximum a posterior probability estimation (MAP)
+// ----------------------------------------------------------------------------|
 void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
-			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
-			 int *b, int *b_star, double D[], double Ds[], double *threshold, int *multi_update )
+			 char *sample_graphs[], double graph_weights[], int *size_sample_g, int *counter_all_g,
+			 int *b, int *b_star, double D[], double Ds[], int *multi_update, int *print )
 {
+	int print_c = *print;
 	int multi_update_C = *multi_update;
 	int iteration = *iter, burn_in = *burnin, b1 = *b;
-	int counterallG = 0;
+	int count_all_g = *counter_all_g;
 	string string_g;
 	vector<string> sample_graphs_C( iteration - burn_in );
 	
 	int selected_edge_i, selected_edge_j, selected_edge_ij, counter, size_sample_graph = *size_sample_g;
 	bool this_one;
 
-	double Dsijj, Dsjj, Dsij, K022, threshold_C = *threshold;
-	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
+	double Dsijj, Dsjj, Dsij, K022, weight_C;
+	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
 	vector<double> sigma( pxp ); 
 	vector<double> copyK( pxp ); 
@@ -617,7 +633,7 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double T
 	int qp = dim * ( dim - 1 ) / 2;
 	vector<char> char_g( qp );              // char string_g[pp];
 	double sum_weights = 0.0;
-	long double rate, sum_rates; 
+	double log_rate, sum_rates; 
 	
 	vector<double> K121( 4 ); 																
 
@@ -631,7 +647,7 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double T
 	}
 
 	// For finding the index of rates 
-	vector<long double> rates( qp );
+	vector<double> rates( qp );
 	vector<int> index_rates_row( qp );
 	vector<int> index_rates_col( qp );
 	counter = 0;
@@ -679,20 +695,18 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double T
 	int size_index = multi_update_C;
 	vector<int> index_selected_edges( multi_update_C );
 
-	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
-
 	GetRNGstate();
 	// main loop for multiple birth-death MCMC sampling algorithm -------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc += size_index )
 	{
-		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
+		if( ( i_mcmc + 1 ) % print_c == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
 		// STEP 1: copula -----------------------------------------------------|
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
 		get_Ts( Ds, Ts, &inv_Ds[0], &copy_Ds[0], &dim );
 		
 		// sampling from K and sigma for double Metropolis-Hastings
-		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
 		counter = 0;
 		// STEP 1: calculating birth and death rates --------------------------|		
@@ -722,14 +736,47 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double T
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
 
-				rate = ( G[ij] ) ? exp( logH_ij - logI_p ) : exp( logI_p - logH_ij );
-				
-				rates[counter] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
+				log_rate = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );				
+				rates[counter] = ( log_rate < 0.0 ) ? exp( log_rate ) : 1.0;			
 				
 				char_g[counter] = G[ij] + '0'; 
 				counter++; 
 			}
 		}
+
+//----- saving result ---------------------------------------------------------|	
+		if( i_mcmc >= burn_in )
+		{
+			weight_C = 1.0 / sum_rates;
+			
+			//for( i = 0; i < pxp; i++ ) K_hat[i] += K[i] / sum_rates;
+			F77_NAME(daxpy)( &pxp, &weight_C, &K[0], &one, &K_hat[0], &one );			
+
+			string_g = string( char_g.begin(), char_g.end() );	
+			all_weights[count_all_g] = weight_C;
+			
+			this_one = false;
+			for( i = 0; i < size_sample_graph; i++ )
+				if( sample_graphs_C[i] == string_g )
+				{
+					graph_weights[i] += all_weights[count_all_g];
+					all_graphs[count_all_g] = i;
+					this_one = true;
+					break;
+				} 
+			
+			if( !this_one || size_sample_graph == 0 )
+			{
+				sample_graphs_C[size_sample_graph] = string_g;
+				graph_weights[size_sample_graph]   = all_weights[count_all_g];
+				all_graphs[count_all_g]            = size_sample_graph; 
+				size_sample_graph++;				
+			}
+			
+			count_all_g++; 
+			sum_weights += weight_C;
+		} 
+//----- End of saving result --------------------------------------------------|	
 		
 		// Selecting multiple edges based on birth and death rates
 		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &qp );
@@ -757,37 +804,7 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double T
 		}
 
 		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
-		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
-
-		// saving result ------------------------------------------------------|	
-		if( i_mcmc >= burn_in )
-		{
-			for( i = 0; i < pxp ; i++ ) K_hat[i] += K[i] / sum_rates;	
-
-			string_g = string( char_g.begin(), char_g.end() );	
-			all_weights[counterallG] = 1.0 / sum_rates;
-			
-			this_one = false;
-			for( i = 0; i < size_sample_graph; i++ )
-				if( sample_graphs_C[i] == string_g )
-				{
-					graph_weights[i] += all_weights[counterallG];
-					all_graphs[counterallG] = i;
-					this_one = true;
-					break;
-				} 
-			
-			if( !this_one || size_sample_graph == 0 )
-			{
-				sample_graphs_C[size_sample_graph] = string_g;
-				graph_weights[size_sample_graph]   = all_weights[counterallG];
-				all_graphs[counterallG]          = size_sample_graph; 
-				size_sample_graph++;				
-			}
-			
-			counterallG++; 
-			sum_weights += 1.0 / sum_rates;
-		} // End of saving result ---------------------------------------------|	
+		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
@@ -798,28 +815,27 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double T
 	}
 	
 	*size_sample_g = size_sample_graph;
+	*counter_all_g = count_all_g;
 
-	for( i = 0; i < pxp; i++ ) 
-		K_hat[i] /= sum_weights;
+	for( i = 0; i < pxp; i++ ) K_hat[i] /= sum_weights;
 }
        
-/*
- * Reversible Jump MCMC for Gaussian copula Graphical models  
- * Based on Double Metropolis-Hastings
- * it is for Bayesian model averaging
-*/
-
-// main function of RJMCMC for GCGMs based on double Metropolis-Hastings
+// ----------------------------------------------------------------------------|
+// Reversible Jump MCMC for Gaussian copula Graphical models  
+// Based on Double Metropolis-Hastings
+// it is for Bayesian model averaging
+// ----------------------------------------------------------------------------|
 void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], int p_links[], 
-			 int *b, int *b_star, double D[], double Ds[], double *threshold )
+			 int *b, int *b_star, double D[], double Ds[], int *print )
 {
+	int print_c = *print;
 	int iteration = *iter, burn_in = *burnin, b1 = *b;
 	
 	int randomEdge, counter, selected_edge_i, selected_edge_j;
 
-	double Dsijj, Dsjj, Dsij, K022, threshold_C = *threshold;
+	double Dsijj, Dsjj, Dsij, K022;
 	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
 
 	vector<double> sigma( pxp ); 
@@ -878,7 +894,7 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 	// main loop for Reversible Jump MCMC sampling algorithm ------------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
 	{
-		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
+		if( ( i_mcmc + 1 ) % print_c == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
 		// STEP 1: copula -----------------------------------------------------| 
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
@@ -912,7 +928,7 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 		Dijj  = - Dij * Dij / Djj;
 		
 		// sampling from K and sigma for double Metropolis-Hastings
-		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
 		log_H_ij( &K[0], &sigma[0], &logH_ij, &selected_edge_i, &selected_edge_j,
                &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
@@ -949,7 +965,7 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 		}
 
 		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
-		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
 		// saving result ------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -963,26 +979,27 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti
 	PutRNGstate();
 }
     
-/*
- * Reversible Jump MCMC for Gaussian copula Graphical models  
- * Based on Double Metropolis-Hastings
- * it is for maximum a posterior probability estimation (MAP)
-*/
+// ----------------------------------------------------------------------------|
+// Reversible Jump MCMC for Gaussian copula Graphical models  
+// Based on Double Metropolis-Hastings
+// it is for maximum a posterior probability estimation (MAP)
+// ----------------------------------------------------------------------------|
 void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
-			 int *b, int *b_star, double D[], double Ds[], double *threshold )
+			 int *b, int *b_star, double D[], double Ds[], int *print )
 {
+	int print_c = *print;
 	int iteration = *iter, burn_in = *burnin, b1 = *b;
-	int counterallG = 0;
+	int count_all_g = 0;
 	string string_g;
 	vector<string> sample_graphs_C( iteration - burn_in );
 	
 	int randomEdge, counter, selected_edge_i, selected_edge_j, size_sample_graph = *size_sample_g;
 	bool this_one;
 
-	double Dsijj, Dsjj, Dsij, K022, threshold_C = *threshold;
+	double Dsijj, Dsjj, Dsij, K022;
 	int i, j, ij, jj, dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
 
 	vector<double> sigma( pxp ); 
@@ -1042,7 +1059,7 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 	// main loop for Reversible Jump MCMC sampling algorithm ------------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
 	{
-		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
+		if( ( i_mcmc + 1 ) % print_c == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
 		// STEP 1: copula -----------------------------------------------------| 
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
@@ -1076,7 +1093,7 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 		Dijj  = - Dij * Dij / Djj;
 		
 		// sampling from K and sigma for double Metropolis-Hastings
-		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
 		log_H_ij( &K[0], &sigma[0], &logH_ij, &selected_edge_i, &selected_edge_j,
                &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
@@ -1111,7 +1128,7 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 		}
 
 		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
-		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
+		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
 		// saving result ------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -1124,8 +1141,8 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 			for( i = 0; i < size_sample_graph; i++ )
 				if( sample_graphs_C[i] == string_g )
 				{
-					graph_weights[i]++;     // += all_weights[counterallG];
-					all_graphs[counterallG] = i;
+					graph_weights[i]++;     // += all_weights[count_all_g];
+					all_graphs[count_all_g] = i;
 					this_one = true;
 					break;
 				} 
@@ -1133,12 +1150,12 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double T
 			if( !this_one || size_sample_graph == 0 )
 			{
 				sample_graphs_C[size_sample_graph] = string_g;
-				graph_weights[size_sample_graph]   = all_weights[counterallG];
-				all_graphs[counterallG]            = size_sample_graph; 
+				graph_weights[size_sample_graph]   = all_weights[count_all_g];
+				all_graphs[count_all_g]            = size_sample_graph; 
 				size_sample_graph++;				
 			}
 			
-			counterallG++; 
+			count_all_g++; 
 		} // End of saving result ---------------------------------------------|	
 	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
